@@ -3,24 +3,27 @@
 """
 
 import itertools
-import ntpath
 import posixpath
 import re
 import warnings
+from collections.abc import Sequence
 from pathlib import Path, PurePath
-from typing import Optional, Pattern, Sequence, Tuple
+from re import Pattern
+from typing import Final, Optional
 
 from ._base import AbstractSanitizer, AbstractValidator, BaseFile, BaseValidator
-from ._common import findall_to_str, to_str, validate_pathtype
+from ._common import findall_to_str, is_nt_abspath, to_str, truncate_str, validate_pathtype
 from ._const import DEFAULT_MIN_LEN, INVALID_CHAR_ERR_MSG_TMPL, Platform
 from ._types import PathType, PlatformType
 from .error import ErrorAttrKey, ErrorReason, InvalidCharError, ValidationError
 from .handler import ReservedNameHandler, ValidationErrorHandler
 
 
-_DEFAULT_MAX_FILENAME_LEN = 255
-_RE_INVALID_FILENAME = re.compile(f"[{re.escape(BaseFile._INVALID_FILENAME_CHARS):s}]", re.UNICODE)
-_RE_INVALID_WIN_FILENAME = re.compile(
+_DEFAULT_MAX_FILENAME_LEN: Final = 255
+_RE_INVALID_FILENAME: Final = re.compile(
+    f"[{re.escape(BaseFile._INVALID_FILENAME_CHARS):s}]", re.UNICODE
+)
+_RE_INVALID_WIN_FILENAME: Final = re.compile(
     f"[{re.escape(BaseFile._INVALID_WIN_FILENAME_CHARS):s}]", re.UNICODE
 )
 
@@ -55,7 +58,6 @@ class FileNameSanitizer(AbstractSanitizer):
             null_value_handler=null_value_handler,
             reserved_name_handler=reserved_name_handler,
             additional_reserved_names=additional_reserved_names,
-            platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
             platform=platform,
             validate_after_sanitize=validate_after_sanitize,
             validator=fname_validator,
@@ -71,11 +73,11 @@ class FileNameSanitizer(AbstractSanitizer):
                 if isinstance(value, PurePath):
                     raise
 
-                return self._null_value_handler(e)
+                return self._null_value_handler(e)  # type: ignore
             raise
 
         sanitized_filename = self._sanitize_regexp.sub(replacement_text, str(value))
-        sanitized_filename = sanitized_filename[: self.max_len]
+        sanitized_filename = truncate_str(sanitized_filename, self._fs_encoding, self.max_len)
 
         try:
             self._validator.validate(sanitized_filename)
@@ -110,9 +112,9 @@ class FileNameSanitizer(AbstractSanitizer):
                 )
 
         if isinstance(value, PurePath):
-            return Path(sanitized_filename)
+            return Path(sanitized_filename)  # type: ignore
 
-        return sanitized_filename
+        return sanitized_filename  # type: ignore
 
     def _get_sanitize_regexp(self) -> Pattern[str]:
         if self._is_windows(include_universal=True):
@@ -122,13 +124,21 @@ class FileNameSanitizer(AbstractSanitizer):
 
 
 class FileNameValidator(BaseValidator):
-    _WINDOWS_RESERVED_FILE_NAMES = ("CON", "PRN", "AUX", "CLOCK$", "NUL") + tuple(
-        f"{name:s}{num:d}" for name, num in itertools.product(("COM", "LPT"), range(1, 10))
+    _WINDOWS_RESERVED_FILE_NAMES: Final = (
+        ("CON", "PRN", "AUX", "CLOCK$", "NUL")
+        + tuple(f"{name:s}{num:d}" for name, num in itertools.product(("COM", "LPT"), range(0, 10)))
+        + tuple(
+            f"{name:s}{ssd:s}"
+            for name, ssd in itertools.product(
+                ("COM", "LPT"),
+                ("\N{SUPERSCRIPT ONE}", "\N{SUPERSCRIPT TWO}", "\N{SUPERSCRIPT THREE}"),
+            )
+        )
     )
-    _MACOS_RESERVED_FILE_NAMES = (":",)
+    _MACOS_RESERVED_FILE_NAMES: Final = (":",)
 
     @property
-    def reserved_keywords(self) -> Tuple[str, ...]:
+    def reserved_keywords(self) -> tuple[str, ...]:
         common_keywords = super().reserved_keywords
 
         if self._is_universal():
@@ -161,7 +171,6 @@ class FileNameValidator(BaseValidator):
             fs_encoding=fs_encoding,
             check_reserved=check_reserved,
             additional_reserved_names=additional_reserved_names,
-            platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
             platform=platform,
         )
 
@@ -208,7 +217,7 @@ class FileNameValidator(BaseValidator):
         )
 
         if self._is_windows(include_universal=True):
-            if ntpath.isabs(value):
+            if is_nt_abspath(value):
                 raise err
 
         if posixpath.isabs(value):
@@ -293,10 +302,10 @@ def validate_filename(
 
             Defaults to ``255``.
         fs_encoding:
-            Filesystem encoding that used to calculate the byte length of the filename.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the filename.
+            If |None|, get the encoding from the execution environment.
         check_reserved:
-            If |True|, check reserved names of the ``platform``.
+            If |True|, check the reserved names of the ``platform``.
         additional_reserved_names:
             Additional reserved names to check.
             Case insensitive.
@@ -310,7 +319,7 @@ def validate_filename(
             The following characters are also invalid for Windows platforms:
             |invalid_win_filename_chars|.
         ValidationError (ErrorReason.RESERVED_NAME):
-            If the ``filename`` equals reserved name by OS.
+            If the ``filename`` equals the reserved name by OS.
             Windows reserved name is as follows:
             ``"CON"``, ``"PRN"``, ``"AUX"``, ``"NUL"``, ``"COM[1-9]"``, ``"LPT[1-9]"``.
 
@@ -405,8 +414,8 @@ def sanitize_filename(
             Truncate the name length if the ``filename`` length exceeds this value.
             Defaults to ``255``.
         fs_encoding:
-            Filesystem encoding that used to calculate the byte length of the filename.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the filename.
+            If |None|, get the encoding from the execution environment.
         check_reserved:
             [Deprecated] Use 'reserved_name_handler' instead.
         null_value_handler:
