@@ -1,3 +1,5 @@
+# type: ignore
+
 """
 .. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
 """
@@ -34,6 +36,9 @@ from ._common import (
     VALID_PLATFORM_NAMES,
     WIN_RESERVED_FILE_NAMES,
     is_faker_installed,
+    platform_linux,
+    platform_macos,
+    platform_windows,
     randstr,
 )
 
@@ -57,11 +62,11 @@ class Test_FileSanitizer:
     )
     def test_normal_platform_auto(self, monkeypatch, test_platform, expected):
         if test_platform == "windows":
-            patch = lambda: "windows"
+            patch = platform_windows
         elif test_platform == "linux":
-            patch = lambda: "linux"
+            patch = platform_linux
         elif test_platform == "macos":
-            patch = lambda: "macos"
+            patch = platform_macos
         else:
             raise ValueError(f"unexpected test platform: {test_platform}")
 
@@ -83,6 +88,7 @@ class Test_FileNameValidator:
                 (
                     "AUX",
                     "CLOCK$",
+                    "COM0",
                     "COM1",
                     "COM2",
                     "COM3",
@@ -92,7 +98,11 @@ class Test_FileNameValidator:
                     "COM7",
                     "COM8",
                     "COM9",
+                    "COM²",
+                    "COM³",
+                    "COM¹",
                     "CON",
+                    "LPT0",
                     "LPT1",
                     "LPT2",
                     "LPT3",
@@ -102,6 +112,9 @@ class Test_FileNameValidator:
                     "LPT7",
                     "LPT8",
                     "LPT9",
+                    "LPT²",
+                    "LPT³",
+                    "LPT¹",
                     "NUL",
                     "PRN",
                 ),
@@ -174,7 +187,7 @@ class Test_validate_filename:
         value = "  "
         with pytest.raises(ValidationError) as e:
             validate_filename(value, platform=platform)
-            assert e.value.reason == ErrorReason.NULL
+            assert e.value.reason == ErrorReason.NULL_NAME
         assert not is_valid_filename(value, platform)
 
     @pytest.mark.parametrize(
@@ -357,6 +370,12 @@ class Test_validate_filename:
                 WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
             )
         ]
+        + [
+            [f"{reserved_keyword}.tar.gz", platform, ValidationError]
+            for reserved_keyword, platform in product(
+                WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
+            )
+        ]
         + [
             [reserved_keyword, platform, None]
             for reserved_keyword, platform in product([".", ".."], ["posix", "linux", "macos"])
@@ -386,34 +405,40 @@ class Test_validate_filename:
             ["Abc", ["abc"], False],
             ["ABC", ["abc"], False],
             ["abc.txt", ["abc.txt"], False],
+            [".", [".", ".."], False],
+            ["..", [".", ".."], False],
         ],
     )
     def test_normal_additional_reserved_names(self, value, arn, expected):
         assert is_valid_filename(value, additional_reserved_names=arn) == expected
 
     @pytest.mark.parametrize(
-        ["platform", "value", "expected"],
+        ["platform", "value", "expected", "reason"],
         [
-            [win_abspath, platform, None]
+            [win_abspath, platform, None, None]
             for win_abspath, platform in product(
                 ["linux", "macos", "posix"],
                 ["\\", "\\\\", "\\ ", "C:\\", "c:\\", "\\xyz", "\\xyz "],
             )
         ]
         + [
-            [win_abspath, platform, ValidationError]
+            [win_abspath, platform, ValidationError, ErrorReason.FOUND_ABS_PATH]
+            for win_abspath, platform in product(["windows", "universal"], ["\\\\", "C:\\", "c:\\"])
+        ]
+        + [
+            [win_abspath, platform, ValidationError, ErrorReason.INVALID_CHARACTER]
             for win_abspath, platform in product(
-                ["windows", "universal"], ["\\", "\\\\", "\\ ", "C:\\", "c:\\", "\\xyz", "\\xyz "]
+                ["windows", "universal"], ["\\", "\\ ", "\\xyz", "\\xyz "]
             )
         ],
     )
-    def test_win_abs_path(self, platform, value, expected):
+    def test_win_abs_path(self, platform, value, expected, reason):
         if expected is None:
             validate_filename(value, platform=platform)
         else:
             with pytest.raises(expected) as e:
                 validate_filename(value, platform=platform)
-            assert e.value.reason == ErrorReason.FOUND_ABS_PATH
+            assert e.value.reason == reason
 
     @pytest.mark.parametrize(
         ["value", "platform"],
@@ -460,21 +485,6 @@ class Test_validate_filename:
             validate_filename(value)
         assert not is_valid_filename(value)
 
-    @pytest.mark.parametrize(
-        ["value", "expected"],
-        [
-            ["a" * 256, ValidationError],
-            [1, TypeError],
-            [True, TypeError],
-            [nan, TypeError],
-            [inf, TypeError],
-        ],
-    )
-    def test_exception(self, value, expected):
-        with pytest.raises(expected):
-            validate_filename(value)
-        assert not is_valid_filename(value)
-
 
 class Test_sanitize_filename:
     SANITIZE_CHARS = INVALID_WIN_FILENAME_CHARS + unprintable_ascii_chars
@@ -646,6 +656,35 @@ class Test_sanitize_filename:
                 == expected
             )
 
+    @pytest.mark.parametrize(
+        ["value", "test_platform", "arn", "expected"],
+        [
+            [".", "windows", [".", ".."], "._"],
+            [".", "universal", [".", ".."], "._"],
+            ["..", "windows", [".", ".."], ".._"],
+            ["..", "universal", [".", ".."], ".._"],
+            ["...", "linux", [".", ".."], "..."],
+        ],
+    )
+    def test_normal_custom_reserved_name_handler_for_dot_files(
+        self, value, test_platform, arn, expected
+    ):
+        def always_add_trailing_underscore(e: ValidationError) -> str:
+            if e.reusable_name:
+                return e.reserved_name
+
+            return f"{e.reserved_name}_"
+
+        assert (
+            sanitize_filename(
+                value,
+                platform=test_platform,
+                reserved_name_handler=always_add_trailing_underscore,
+                additional_reserved_names=arn,
+            )
+            == expected
+        )
+
     def test_exception_reserved_name_handler(self):
         for platform in ["windows", "universal"]:
             with pytest.raises(ValidationError) as e:
@@ -668,7 +707,7 @@ class Test_sanitize_filename:
                     additional_reserved_names=arn,
                 )
                 == expected
-            )
+            ), platform
 
     @pytest.mark.parametrize(
         ["value", "check_reserved", "expected"],
@@ -701,6 +740,7 @@ class Test_sanitize_filename:
             ["linux", "period.", "period."],
             ["linux", "space ", "space "],
             ["linux", "space_and_period. ", "space_and_period. "],
+            ["linux", "...", "..."],
             ["universal", "period.", "period"],
             ["universal", "space ", "space"],
             ["universal", "space_and_period .", "space_and_period"],
@@ -751,3 +791,20 @@ class Test_sanitize_filename:
         with pytest.raises(expected):
             sanitize_filename(value)
         assert not is_valid_filename(value)
+
+    @pytest.mark.parametrize(
+        ["value", "platform", "fs_encoding", "max_len", "expected"],
+        [
+            ["あ" * 85, "universal", "utf-8", 255, "あ" * 85],
+            ["あ" * 86, "universal", "utf-8", 255, "あ" * 85],
+            ["あ" * 126, "universal", "utf-16", 255, "あ" * 126],
+            ["あ" * 127, "universal", "utf-16", 255, "あ" * 126],
+        ],
+    )
+    def test_max_len_fs_encoding(self, value, platform, fs_encoding, max_len, expected):
+        kwargs = {
+            "platform": platform,
+            "max_len": max_len,
+            "fs_encoding": fs_encoding,
+        }
+        assert sanitize_filename(value, **kwargs) == expected
