@@ -1,3 +1,5 @@
+# type: ignore
+
 """
 .. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
 """
@@ -31,6 +33,9 @@ from ._common import (
     VALID_PATH_CHARS,
     WIN_RESERVED_FILE_NAMES,
     is_faker_installed,
+    platform_linux,
+    platform_macos,
+    platform_windows,
     randstr,
 )
 
@@ -52,11 +57,11 @@ class Test_FileSanitizer:
     )
     def test_normal_platform_auto(self, monkeypatch, test_platform, expected):
         if test_platform == "windows":
-            patch = lambda: "windows"
+            patch = platform_windows
         elif test_platform == "linux":
-            patch = lambda: "linux"
+            patch = platform_linux
         elif test_platform == "macos":
-            patch = lambda: "macos"
+            patch = platform_macos
         else:
             raise ValueError(f"unexpected test platform: {test_platform}")
 
@@ -101,6 +106,7 @@ class Test_validate_filepath:
         "C:\\Users",
         "C:\\",
         "\\Users",
+        r"\\fileserver\public",
     ]
 
     @pytest.mark.parametrize(
@@ -185,6 +191,7 @@ class Test_validate_filepath:
             validate_filepath(value, min_len=min_len)
         assert e.value.reason == expected
         assert e.value.fs_encoding
+        assert e.value.byte_count
         assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
@@ -222,6 +229,7 @@ class Test_validate_filepath:
             validate_filepath(value, **kwargs)
         assert e.value.reason == ErrorReason.INVALID_LENGTH
         assert e.value.fs_encoding
+        assert e.value.byte_count
         assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
@@ -249,6 +257,7 @@ class Test_validate_filepath:
             validate_filepath(value, **kwargs)
         assert e.value.reason == expected
         assert e.value.fs_encoding
+        assert e.value.byte_count
         assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
@@ -281,7 +290,7 @@ class Test_validate_filepath:
         [
             ["linux", "/a/b/c.txt", None],
             ["linux", "C:\\a\\b\\c.txt", ValidationError],
-            ["windows", "/a/b/c.txt", None],
+            ["windows", "/a/b/c.txt", ValidationError],
             ["windows", "C:\\a\\b\\c.txt", None],
             ["universal", "/a/b/c.txt", ValidationError],
             ["universal", "C:\\a\\b\\c.txt", ValidationError],
@@ -355,24 +364,40 @@ class Test_validate_filepath:
     @pytest.mark.parametrize(
         ["platform", "value"],
         [
-            ["windows", "period."],
-            ["windows", "space "],
-            ["windows", "space_and_period ."],
-            ["windows", "space_and_period. "],
             ["linux", "period."],
             ["linux", "space "],
             ["linux", "space_and_period. "],
-            ["universal", "period."],
-            ["universal", "space "],
-            ["universal", "space_and_period ."],
         ],
     )
     def test_normal_space_or_period_at_tail(self, platform, value):
         validate_filepath(value, platform=platform)
         assert is_valid_filepath(value, platform=platform)
 
+    @pytest.mark.parametrize(
+        ["platform", "value"],
+        [
+            ["windows", "period."],
+            ["windows", "space "],
+            ["windows", "space_and_period ."],
+            ["windows", "space_and_period. "],
+            ["universal", "period."],
+            ["universal", "space "],
+            ["universal", "space_and_period ."],
+        ],
+    )
+    def test_exception_space_or_period_at_tail(self, platform, value):
+        with pytest.raises(ValidationError):
+            validate_filepath(value, platform=platform)
+        assert not is_valid_filepath(value, platform=platform)
+
     @pytest.mark.skipif(not is_faker_installed(), reason="requires faker")
-    @pytest.mark.parametrize(["locale"], [[None], ["ja_JP"]])
+    @pytest.mark.parametrize(
+        ["locale"],
+        [
+            [None],
+            ["ja_JP"],
+        ],
+    )
     def test_locale_jp(self, locale):
         from faker import Factory
 
@@ -676,7 +701,9 @@ class Test_sanitize_filepath:
         ["value", "reserved_name_handler", "expected"],
         [
             ["CON", ReservedNameHandler.add_trailing_underscore, "CON_"],
+            ["hoge/CON", ReservedNameHandler.add_trailing_underscore, "hoge\\CON_"],
             ["CON", ReservedNameHandler.add_leading_underscore, "_CON"],
+            ["hoge/CON", ReservedNameHandler.add_leading_underscore, "hoge\\_CON"],
             ["CON", ReservedNameHandler.as_is, "CON"],
         ],
     )
@@ -688,6 +715,32 @@ class Test_sanitize_filepath:
             == expected
         )
 
+    @pytest.mark.parametrize(
+        ["value", "expected"],
+        [
+            ["hoge/.", "hoge\\._"],
+            ["hoge/./foo", "hoge\\._\\foo"],
+            ["hoge/..", "hoge\\.._"],
+        ],
+    )
+    def test_normal_custom_reserved_name_handler_for_dot_files(self, value, expected):
+        def always_add_trailing_underscore(e: ValidationError) -> str:
+            if e.reusable_name:
+                return e.reserved_name
+
+            return f"{e.reserved_name}_"
+
+        assert (
+            sanitize_filepath(
+                value,
+                platform="windows",
+                reserved_name_handler=always_add_trailing_underscore,
+                additional_reserved_names=[".", ".."],
+                normalize=False,
+            )
+            == expected
+        )
+
     def test_exception_reserved_name_handler(self):
         for platform in ["windows", "universal"]:
             with pytest.raises(ValidationError) as e:
